@@ -1,3 +1,29 @@
+/* GDK - The GIMP Drawing Kit
+ * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+/*
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * file for a list of people on the GTK+ Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
 #ifndef __GDK_COLOR_H__
 #define __GDK_COLOR_H__
 
@@ -101,13 +127,14 @@ void     gdk_colormap_query_color (GdkColormap *colormap,
 
 GdkVisual *gdk_colormap_get_visual (GdkColormap *colormap);
      
-GdkColor *gdk_color_copy  (const GdkColor *color);
-void      gdk_color_free  (GdkColor       *color);
-gint      gdk_color_parse (const gchar    *spec,
-			   GdkColor       *color);
-guint     gdk_color_hash  (const GdkColor *colora);
-gboolean  gdk_color_equal (const GdkColor *colora,
-			   const GdkColor *colorb);
+GdkColor *gdk_color_copy      (const GdkColor *color);
+void      gdk_color_free      (GdkColor       *color);
+gboolean  gdk_color_parse     (const gchar    *spec,
+			       GdkColor       *color);
+guint     gdk_color_hash      (const GdkColor *colora);
+gboolean  gdk_color_equal     (const GdkColor *colora,
+			       const GdkColor *colorb);
+gchar *   gdk_color_to_string (const GdkColor *color);
 
 GType     gdk_color_get_type (void) G_GNUC_CONST;
 
