@@ -39,6 +39,13 @@
   return NO;
 }
 
+-(void)windowWillMiniaturize:(NSNotification *)aNotification
+{
+  GdkWindow *window = [[self contentView] gdkWindow];
+
+  _gdk_quartz_window_detach_from_parent (window);
+}
+
 -(void)windowDidMiniaturize:(NSNotification *)aNotification
 {
   GdkWindow *window = [[self contentView] gdkWindow];
@@ -51,6 +58,8 @@
 {
   GdkWindow *window = [[self contentView] gdkWindow];
 
+  _gdk_quartz_window_attach_to_parent (window);
+
   gdk_synthesize_window_state (window, GDK_WINDOW_STATE_ICONIFIED, 0);
 }
 
@@ -68,6 +77,22 @@
   _gdk_quartz_events_update_focus_window (window, FALSE);
 }
 
+-(void)windowDidBecomeMain:(NSNotification *)aNotification
+{
+  GdkWindow *window;
+
+  window = [[self contentView] gdkWindow];
+  _gdk_quartz_window_did_become_main (window);
+}
+
+-(void)windowDidResignMain:(NSNotification *)aNotification
+{
+  GdkWindow *window;
+
+  window = [[self contentView] gdkWindow];
+  _gdk_quartz_window_did_resign_main (window);
+}
+
 /* Used in combination with NSLeftMouseUp in sendEvent to keep track
  * of when the window is being moved with the mouse.
  */
@@ -135,8 +160,9 @@
   impl->width = content_rect.size.width;
   impl->height = content_rect.size.height;
 
-  /* Synthesize a configure event */
+  [[self contentView] setBounds:NSMakeRect (0, 0, impl->width, impl->height)];
 
+  /* Synthesize a configure event */
   event = gdk_event_new (GDK_CONFIGURE);
   event->configure.window = g_object_ref (window);
   event->configure.x = private->x;
@@ -145,13 +171,6 @@
   event->configure.height = impl->height;
 
   _gdk_event_queue_append (gdk_display_get_default (), event);
-
-  /* Update tracking rectangle */
-  [[self contentView] removeTrackingRect:impl->tracking_rect];
-  impl->tracking_rect = [impl->view addTrackingRect:NSMakeRect(0, 0, impl->width, impl->height) 
-			                      owner:impl->view
-			                   userData:nil
-			               assumeInside:NO];
 }
 
 -(id)initWithContentRect:(NSRect)contentRect styleMask:(unsigned int)styleMask backing:(NSBackingStoreType)backingType defer:(BOOL)flag
@@ -161,12 +180,7 @@
 	                    backing:backingType
 	                      defer:flag];
 
-
-  /* A possible modification here would be to only accept mouse moved events
-   * if any of the child GdkWindows are interested in mouse moved events.
-   */
   [self setAcceptsMouseMovedEvents:YES];
-
   [self setDelegate:self];
   [self setReleasedWhenClosed:YES];
 
