@@ -355,5 +355,5 @@ static const struct { const gchar *name; const gchar *builtin; gint type; guchar
     "\125\025\132\232\225\152\124\026\120\001\220\000\144\000\031\000"
     "\006\100\001\220\000\144\000\031\000\006\100\005\224\025\251\126"
     "\246\245\124\125" }, 
-  { NULL, 0, 0, 0, 0, 0, NULL },
+  { NULL, NULL, 0, 0, 0, 0, 0, NULL },
 };
