@@ -40,6 +40,8 @@
 
 /* --- prototypes --- */
 static void gtk_accel_group_finalize	(GObject		*object);
+static void accel_closure_invalidate    (gpointer  data,
+                                         GClosure *closure);
 
 
 /* --- variables --- */
@@ -134,6 +136,7 @@ gtk_accel_group_finalize (GObject *object)
 
 	  _gtk_accel_map_remove_group (accel_path, accel_group);
 	}
+      g_closure_remove_invalidate_notifier (entry->closure, accel_group, accel_closure_invalidate);
     }
 
   g_free (accel_group->priv_accels);
