@@ -42,8 +42,8 @@
 #include "gtkalias.h"
 
 /* Do *not* include "gtkprivate.h" in this file. If you do, the
- * correct_libdir_prefix() function below will have to move somewhere
- * else.
+ * correct_libdir_prefix() and correct_localedir_prefix() functions
+ * below will have to move somewhere else.
  */
 
 #ifdef __GTK_PRIVATE_H__
@@ -95,7 +95,7 @@ gtk_im_module_load (GTypeModule *module)
 {
   GtkIMModule *im_module = GTK_IM_MODULE (module);
   
-  im_module->library = g_module_open (im_module->path, 0);
+  im_module->library = g_module_open (im_module->path, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
   if (!im_module->library)
     {
       g_warning (g_module_error());
@@ -154,6 +154,8 @@ gtk_im_module_finalize (GObject *object)
   parent_class->finalize (object);
 }
 
+G_DEFINE_TYPE (GtkIMModule, gtk_im_module, G_TYPE_TYPE_MODULE)
+
 static void
 gtk_im_module_class_init (GtkIMModuleClass *class)
 {
@@ -168,31 +170,9 @@ gtk_im_module_class_init (GtkIMModuleClass *class)
   gobject_class->finalize = gtk_im_module_finalize;
 }
 
-static GType
-gtk_im_module_get_type (void)
+static void 
+gtk_im_module_init (GtkIMModule* object)
 {
-  static GType im_module_type = 0;
-
-  if (!im_module_type)
-    {
-      static const GTypeInfo im_module_info = {
-        sizeof (GtkIMModuleClass),
-        NULL,           /* base_init */
-        NULL,           /* base_finalize */
-        (GClassInitFunc) gtk_im_module_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (GtkIMModule),
-        0,              /* n_preallocs */
-        NULL,           /* instance_init */
-      };
-
-      im_module_type =
-	g_type_register_static (G_TYPE_TYPE_MODULE, "GtkIMModule",
-				&im_module_info, 0);
-    }
-  
-  return im_module_type;
 }
 
 static void
@@ -237,12 +217,7 @@ add_module (GtkIMModule *module, GSList *infos)
   modules_list = g_slist_prepend (modules_list, module);
 }
 
-#if defined (G_OS_WIN32) && defined (GTK_LIBDIR)
-/* This is needes on Win32, but not wanted when compiling with MSVC,
- * as the makefile.msc doesn't define any GTK_LIBDIR value.
- */
-
-#define DO_CORRECT_LIBDIR_PREFIX /* Flag to check below whether to call this */
+#ifdef G_OS_WIN32
 
 static void
 correct_libdir_prefix (gchar **path)
@@ -267,11 +242,26 @@ correct_libdir_prefix (gchar **path)
       g_free (tem);
     }
 }
+
+static void
+correct_localedir_prefix (gchar **path)
+{
+  /* As above, but for GTK_LOCALEDIR. Use separate function in case
+   * GTK_LOCALEDIR isn't a subfolder of GTK_LIBDIR.
+   */
+  if (strncmp (*path, GTK_LOCALEDIR, strlen (GTK_LOCALEDIR)) == 0)
+    {
+      extern const gchar *_gtk_get_localedir ();
+      gchar *tem = *path;
+      *path = g_strconcat (_gtk_get_localedir (), tem + strlen (GTK_LOCALEDIR), NULL);
+      g_free (tem);
+    }
+}
 #endif
 
 
 static void
-gtk_im_module_init (void)
+gtk_im_module_initialize (void)
 {
   GString *line_buf = g_string_new (NULL);
   GString *tmp_buf = g_string_new (NULL);
@@ -331,7 +321,7 @@ gtk_im_module_init (void)
 	    }
 
 	  module->path = g_strdup (tmp_buf->str);
-#ifdef DO_CORRECT_LIBDIR_PREFIX
+#ifdef G_OS_WIN32
 	  correct_libdir_prefix (&module->path);
 #endif
 	  g_type_module_set_name (G_TYPE_MODULE (module), module->path);
@@ -357,8 +347,8 @@ gtk_im_module_init (void)
 	  if (!pango_scan_string (&p, tmp_buf))
 	    goto context_error;
 	  info->domain_dirname = g_strdup (tmp_buf->str);
-#ifdef DO_CORRECT_LIBDIR_PREFIX
-	  correct_libdir_prefix ((char **) &info->domain_dirname);
+#ifdef G_OS_WIN32
+	  correct_localedir_prefix ((char **) &info->domain_dirname);
 #endif
 
 	  if (!pango_scan_string (&p, tmp_buf))
@@ -422,7 +412,11 @@ _gtk_im_module_list (const GtkIMContextInfo ***contexts,
 {
   int n = 0;
 
-  static const GtkIMContextInfo simple_context_info = {
+  static
+#ifndef G_OS_WIN32
+	  const
+#endif
+		GtkIMContextInfo simple_context_info = {
     SIMPLE_ID,
     N_("Default"),
     GETTEXT_PACKAGE,
@@ -434,8 +428,24 @@ _gtk_im_module_list (const GtkIMContextInfo ***contexts,
     ""
   };
 
+#ifdef G_OS_WIN32
+  static gboolean beenhere = FALSE;
+#endif
+
   if (!contexts_hash)
-    gtk_im_module_init ();
+    gtk_im_module_initialize ();
+
+#ifdef G_OS_WIN32
+  if (!beenhere)
+    {
+      beenhere = TRUE;
+      /* correct_localedir_prefix() requires its parameter to be a
+       * malloced string
+       */
+      simple_context_info.domain_dirname = g_strdup (simple_context_info.domain_dirname);
+      correct_localedir_prefix ((char **) &simple_context_info.domain_dirname);
+    }
+#endif
 
   if (n_contexts)
     *n_contexts = (n_loaded_contexts + 1);
@@ -482,7 +492,7 @@ _gtk_im_module_create (const gchar *context_id)
   GtkIMContext *context = NULL;
   
   if (!contexts_hash)
-    gtk_im_module_init ();
+    gtk_im_module_initialize ();
 
   if (strcmp (context_id, SIMPLE_ID) != 0)
     {
@@ -556,7 +566,7 @@ _gtk_im_module_get_default_context_id (const gchar *locale)
   const gchar *envvar;
       
   if (!contexts_hash)
-    gtk_im_module_init ();
+    gtk_im_module_initialize ();
 
   envvar = g_getenv ("GTK_IM_MODULE");
   if (envvar &&
