@@ -237,7 +237,7 @@ find_module (const gchar *name)
       module_name = g_module_build_path (NULL, name);
     }
 
-  module = g_module_open (module_name, G_MODULE_BIND_LAZY);
+  module = g_module_open (module_name, G_MODULE_BIND_LOCAL | G_MODULE_BIND_LAZY);
   g_free(module_name);
 
   return module;
