@@ -46,13 +46,7 @@
 #include "gtkobject.h"
 #include "gtktooltips.h"
 #include "gtktypebuiltins.h"
-
-#include "gtkrecentmanager.h"
-#include "gtkrecentfilter.h"
-#include "gtkrecentchooser.h"
-#include "gtkrecentchooserutils.h"
-#include "gtkrecentchooserprivate.h"
-#include "gtkrecentchoosermenu.h"
+#include "gtkprivate.h"
 #include "gtkalias.h"
 
 struct _GtkRecentChooserMenuPrivate
@@ -66,6 +60,9 @@ struct _GtkRecentChooserMenuPrivate
   /* max size of the menu item label */
   gint label_width;
 
+  gint first_recent_item_pos;
+  GtkWidget *placeholder;
+
   /* RecentChooser properties */
   gint limit;  
   guint show_private : 1;
@@ -194,8 +191,8 @@ gtk_recent_chooser_menu_class_init (GtkRecentChooserMenuClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->constructor = gtk_recent_chooser_menu_constructor;
-  gobject_class->finalize = gtk_recent_chooser_menu_finalize;
   gobject_class->dispose = gtk_recent_chooser_menu_dispose;
+  gobject_class->finalize = gtk_recent_chooser_menu_finalize;
   gobject_class->set_property = gtk_recent_chooser_menu_set_property;
   gobject_class->get_property = gtk_recent_chooser_menu_get_property;
 
@@ -215,7 +212,7 @@ gtk_recent_chooser_menu_class_init (GtkRecentChooserMenuClass *klass)
 							 P_("Show Numbers"),
 							 P_("Whether the items should be displayed with a number"),
 							 FALSE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
   
   g_type_class_add_private (klass, sizeof (GtkRecentChooserMenuPrivate));
 }
@@ -237,13 +234,13 @@ gtk_recent_chooser_menu_init (GtkRecentChooserMenu *menu)
   priv->local_only = TRUE;
   
   priv->limit = FALLBACK_ITEM_LIMIT;
-
   priv->sort_type = GTK_RECENT_SORT_NONE;
-  
   priv->icon_size = FALLBACK_ICON_SIZE;
-
   priv->label_width = DEFAULT_LABEL_WIDTH;
   
+  priv->first_recent_item_pos = -1;
+  priv->placeholder = NULL;
+
   priv->current_filter = NULL;
     
   priv->tooltips = gtk_tooltips_new ();
@@ -263,7 +260,7 @@ gtk_recent_chooser_menu_finalize (GObject *object)
       priv->sort_data_destroy (priv->sort_data);
       priv->sort_data_destroy = NULL;
     }
-  
+
   priv->sort_data = NULL;
   priv->sort_func = NULL;
   
@@ -283,7 +280,7 @@ gtk_recent_chooser_menu_dispose (GObject *object)
 
       priv->manager_changed_id = 0;
     }
-
+  
   if (priv->populate_id)
     {
       g_source_remove (priv->populate_id);
@@ -295,31 +292,55 @@ gtk_recent_chooser_menu_dispose (GObject *object)
       g_object_unref (priv->tooltips);
       priv->tooltips = NULL;
     }
-
+ 
   if (priv->current_filter)
     {
       g_object_unref (priv->current_filter);
       priv->current_filter = NULL;
     }
-
+  
   G_OBJECT_CLASS (gtk_recent_chooser_menu_parent_class)->dispose (object);
 }
 
 static GObject *
 gtk_recent_chooser_menu_constructor (GType                  type,
-				     guint                  n_construct_properties,
-				     GObjectConstructParam *construct_params)
+				     guint                  n_params,
+				     GObjectConstructParam *params)
 {
   GtkRecentChooserMenu *menu;
+  GtkRecentChooserMenuPrivate *priv;
+  GObjectClass *parent_class;
   GObject *object;
   
-  object = G_OBJECT_CLASS (gtk_recent_chooser_menu_parent_class)->constructor (type,
-		  							       n_construct_properties,
-									       construct_params);
+  parent_class = G_OBJECT_CLASS (gtk_recent_chooser_menu_parent_class);
+  object = parent_class->constructor (type, n_params, params);
   menu = GTK_RECENT_CHOOSER_MENU (object);
+  priv = menu->priv;
   
-  g_assert (menu->priv->manager);
-  
+  g_assert (priv->manager);
+
+  /* we create a placeholder menuitem, to be used in case
+   * the menu is empty. this placeholder will stay around
+   * for the entire lifetime of the menu, and we just hide it
+   * when it's not used. we have to do this, and do it here,
+   * because we need a marker for the beginning of the recent
+   * items list, so that we can insert the new items at the
+   * right place when idly populating the menu in case the
+   * user appended or prepended custom menu items to the
+   * recent chooser menu widget.
+   */
+  priv->placeholder = gtk_menu_item_new_with_label (_("No items found"));
+  gtk_widget_set_sensitive (priv->placeholder, FALSE);
+  g_object_set_data (G_OBJECT (priv->placeholder),
+                     "gtk-recent-menu-placeholder",
+                     GINT_TO_POINTER (TRUE));
+
+  gtk_menu_shell_insert (GTK_MENU_SHELL (menu), priv->placeholder, 0);
+  gtk_widget_show (priv->placeholder);
+
+  /* (re)populate the menu */
+  gtk_recent_chooser_menu_populate (menu);
+
   return object;
 }
 
@@ -330,26 +351,27 @@ gtk_recent_chooser_menu_set_property (GObject      *object,
 				      GParamSpec   *pspec)
 {
   GtkRecentChooserMenu *menu = GTK_RECENT_CHOOSER_MENU (object);
+  GtkRecentChooserMenuPrivate *priv = menu->priv;
   
   switch (prop_id)
     {
     case PROP_SHOW_NUMBERS:
-      menu->priv->show_numbers = g_value_get_boolean (value);
+      priv->show_numbers = g_value_get_boolean (value);
       break;
     case GTK_RECENT_CHOOSER_PROP_RECENT_MANAGER:
       set_recent_manager (menu, g_value_get_object (value));
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_PRIVATE:
-      menu->priv->show_private = g_value_get_boolean (value);
+      priv->show_private = g_value_get_boolean (value);
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_NOT_FOUND:
-      menu->priv->show_not_found = g_value_get_boolean (value);
+      priv->show_not_found = g_value_get_boolean (value);
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_TIPS:
       gtk_recent_chooser_menu_set_show_tips (menu, g_value_get_boolean (value));
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_ICONS:
-      menu->priv->show_icons = g_value_get_boolean (value);
+      priv->show_icons = g_value_get_boolean (value);
       break;
     case GTK_RECENT_CHOOSER_PROP_SELECT_MULTIPLE:
       g_warning ("%s: Choosers of type `%s' do not support selecting multiple items.",
@@ -357,10 +379,10 @@ gtk_recent_chooser_menu_set_property (GObject      *object,
                  G_OBJECT_TYPE_NAME (object));
       break;
     case GTK_RECENT_CHOOSER_PROP_LOCAL_ONLY:
-      menu->priv->local_only = g_value_get_boolean (value);
+      priv->local_only = g_value_get_boolean (value);
       break;
     case GTK_RECENT_CHOOSER_PROP_LIMIT:
-      menu->priv->limit = g_value_get_int (value);
+      priv->limit = g_value_get_int (value);
       break;
     case GTK_RECENT_CHOOSER_PROP_SORT_TYPE:
       chooser_set_sort_type (menu, g_value_get_enum (value));
@@ -381,32 +403,33 @@ gtk_recent_chooser_menu_get_property (GObject    *object,
 				      GParamSpec *pspec)
 {
   GtkRecentChooserMenu *menu = GTK_RECENT_CHOOSER_MENU (object);
+  GtkRecentChooserMenuPrivate *priv = menu->priv;
   
   switch (prop_id)
     {
     case PROP_SHOW_NUMBERS:
-      g_value_set_boolean (value, menu->priv->show_numbers);
+      g_value_set_boolean (value, priv->show_numbers);
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_TIPS:
-      g_value_set_boolean (value, menu->priv->show_tips);
+      g_value_set_boolean (value, priv->show_tips);
       break;
     case GTK_RECENT_CHOOSER_PROP_LIMIT:
-      g_value_set_int (value, menu->priv->limit);
+      g_value_set_int (value, priv->limit);
       break;
     case GTK_RECENT_CHOOSER_PROP_LOCAL_ONLY:
-      g_value_set_boolean (value, menu->priv->local_only);
+      g_value_set_boolean (value, priv->local_only);
       break;
     case GTK_RECENT_CHOOSER_PROP_SORT_TYPE:
-      g_value_set_enum (value, menu->priv->sort_type);
+      g_value_set_enum (value, priv->sort_type);
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_PRIVATE:
-      g_value_set_boolean (value, menu->priv->show_private);
+      g_value_set_boolean (value, priv->show_private);
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_NOT_FOUND:
-      g_value_set_boolean (value, menu->priv->show_not_found);
+      g_value_set_boolean (value, priv->show_not_found);
       break;
     case GTK_RECENT_CHOOSER_PROP_SHOW_ICONS:
-      g_value_set_boolean (value, menu->priv->show_icons);
+      g_value_set_boolean (value, priv->show_icons);
       break;
     case GTK_RECENT_CHOOSER_PROP_SELECT_MULTIPLE:
       g_warning ("%s: Choosers of type `%s' do not support selecting multiple items.",
@@ -414,7 +437,7 @@ gtk_recent_chooser_menu_get_property (GObject    *object,
                  G_OBJECT_TYPE_NAME (object));
       break;
     case GTK_RECENT_CHOOSER_PROP_FILTER:
-      g_value_set_object (value, menu->priv->current_filter);
+      g_value_set_object (value, priv->current_filter);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -433,6 +456,7 @@ gtk_recent_chooser_menu_set_current_uri (GtkRecentChooser  *chooser,
   gboolean found = FALSE;
   
   children = gtk_container_get_children (GTK_CONTAINER (menu));
+  
   for (l = children; l != NULL; l = l->next)
     {
       GtkRecentInfo *info;
@@ -448,7 +472,7 @@ gtk_recent_chooser_menu_set_current_uri (GtkRecentChooser  *chooser,
           gtk_menu_shell_activate_item (GTK_MENU_SHELL (menu),
 	                                menu_item,
 					TRUE);
-          found = TRUE;
+	  found = TRUE;
 
 	  break;
 	}
@@ -456,14 +480,14 @@ gtk_recent_chooser_menu_set_current_uri (GtkRecentChooser  *chooser,
 
   g_list_free (children);
   
-  if (!found)
+  if (!found)  
     {
       g_set_error (error, GTK_RECENT_CHOOSER_ERROR,
       		   GTK_RECENT_CHOOSER_ERROR_NOT_FOUND,
       		   _("No recently used resource found with URI `%s'"),
       		   uri);
     }
-
+  
   return found;
 }
 
@@ -565,11 +589,12 @@ gtk_recent_chooser_menu_set_sort_func (GtkRecentChooser  *chooser,
   if (priv->sort_data_destroy)
     {
       priv->sort_data_destroy (priv->sort_data);
-      
-      priv->sort_func = NULL;
-      priv->sort_data = NULL;
       priv->sort_data_destroy = NULL;
     }
+      
+  priv->sort_func = NULL;
+  priv->sort_data = NULL;
+  priv->sort_data_destroy = NULL;
   
   if (sort_func)
     {
@@ -579,45 +604,6 @@ gtk_recent_chooser_menu_set_sort_func (GtkRecentChooser  *chooser,
     }
 }
 
-static gint
-sort_recent_items_mru (GtkRecentInfo *a,
-		       GtkRecentInfo *b,
-		       gpointer       unused)
-{
-  g_assert (a != NULL && b != NULL);
-  
-  return (gtk_recent_info_get_modified (a) < gtk_recent_info_get_modified (b));
-}
-
-static gint
-sort_recent_items_lru (GtkRecentInfo *a,
-		       GtkRecentInfo *b,
-		       gpointer       unused)
-{
-  g_assert (a != NULL && b != NULL);
-  
-  return (gtk_recent_info_get_modified (a) > gtk_recent_info_get_modified (b));
-}
-
-/* our proxy sorting function */
-static gint
-sort_recent_items_proxy (gpointer *a,
-			 gpointer *b,
-			 gpointer  user_data)
-{
-  GtkRecentInfo *info_a = (GtkRecentInfo *) a;
-  GtkRecentInfo *info_b = (GtkRecentInfo *) b;
-  GtkRecentChooserMenu *menu = GTK_RECENT_CHOOSER_MENU (user_data);
-
-  if (menu->priv->sort_func)
-    return (* menu->priv->sort_func) (info_a,
-    				      info_b,
-    				      menu->priv->sort_data);
-  
-  /* fallback */
-  return 0;
-}
-
 static void
 chooser_set_sort_type (GtkRecentChooserMenu *menu,
 		       GtkRecentSortType     sort_type)
@@ -633,66 +619,12 @@ static GList *
 gtk_recent_chooser_menu_get_items (GtkRecentChooser *chooser)
 {
   GtkRecentChooserMenu *menu = GTK_RECENT_CHOOSER_MENU (chooser);
-  GtkRecentChooserMenuPrivate *priv;
-  gint limit;
-  GtkRecentSortType sort_type;
-  GList *items;
-  GCompareDataFunc compare_func;
-  gint length;
-  
-  priv = menu->priv;
-  
-  if (!priv->manager)
-    return NULL;
-  
-  limit = gtk_recent_chooser_get_limit (chooser);
-  if (limit == 0)
-    return NULL;
+  GtkRecentChooserMenuPrivate *priv = menu->priv;
 
-  sort_type = gtk_recent_chooser_get_sort_type (chooser);
-  switch (sort_type)
-    {
-    case GTK_RECENT_SORT_NONE:
-      compare_func = NULL;
-      break;
-    case GTK_RECENT_SORT_MRU:
-      compare_func = (GCompareDataFunc) sort_recent_items_mru;
-      break;
-    case GTK_RECENT_SORT_LRU:
-      compare_func = (GCompareDataFunc) sort_recent_items_lru;
-      break;
-    case GTK_RECENT_SORT_CUSTOM:
-      compare_func = (GCompareDataFunc) sort_recent_items_proxy;
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
-    }
-
-  items = gtk_recent_manager_get_items (priv->manager);
-  if (!items)
-    return NULL;
-  
-  if (compare_func)  
-    items = g_list_sort_with_data (items, compare_func, menu);
- 
-  length = g_list_length (items);
-  if ((limit != -1) && (length > limit))
-    {
-      GList *clamp, *l;
-
-      clamp = g_list_nth (items, limit - 1);
-      if (!clamp)
-        return items;
-
-      l = clamp->next;
-      clamp->next = NULL;      
-      
-      g_list_foreach (l, (GFunc) gtk_recent_info_unref, NULL);
-      g_list_free (l);
-    }
-
-  return items;
+  return _gtk_recent_chooser_get_items (chooser,
+                                        priv->current_filter,
+                                        priv->sort_func,
+                                        priv->sort_data);
 }
 
 static GtkRecentManager *
@@ -763,78 +695,12 @@ gtk_recent_chooser_menu_set_current_filter (GtkRecentChooserMenu *menu,
       priv->current_filter = filter;
       g_object_ref_sink (priv->current_filter);
     }
+
+  gtk_recent_chooser_menu_populate (menu);
   
   g_object_notify (G_OBJECT (menu), "filter");
 }
 
-static gboolean
-get_is_recent_filtered (GtkRecentChooserMenu *menu,
-			GtkRecentInfo        *info)
-{
-  GtkRecentChooserMenuPrivate *priv;
-  GtkRecentFilter *current_filter;
-  GtkRecentFilterInfo filter_info;
-  GtkRecentFilterFlags needed;
-  gboolean retval;
-
-  g_assert (info != NULL);
-
-  priv = menu->priv;
-  
-  if (!priv->current_filter)
-    return FALSE;
-  
-  current_filter = priv->current_filter;
-  needed = gtk_recent_filter_get_needed (current_filter);
-  
-  filter_info.contains = GTK_RECENT_FILTER_URI | GTK_RECENT_FILTER_MIME_TYPE;
-  
-  filter_info.uri = gtk_recent_info_get_uri (info);
-  filter_info.mime_type = gtk_recent_info_get_mime_type (info);
-  
-  if (needed & GTK_RECENT_FILTER_DISPLAY_NAME)
-    {
-      filter_info.display_name = gtk_recent_info_get_display_name (info);
-      filter_info.contains |= GTK_RECENT_FILTER_DISPLAY_NAME;
-    }
-  else
-    filter_info.uri = NULL;
-  
-  if (needed & GTK_RECENT_FILTER_APPLICATION)
-    {
-      filter_info.applications = (const gchar **) gtk_recent_info_get_applications (info, NULL);
-      filter_info.contains |= GTK_RECENT_FILTER_APPLICATION;
-    }
-  else
-    filter_info.applications = NULL;
-
-  if (needed & GTK_RECENT_FILTER_GROUP)
-    {
-      filter_info.groups = (const gchar **) gtk_recent_info_get_groups (info, NULL);
-      filter_info.contains |= GTK_RECENT_FILTER_GROUP;
-    }
-  else
-    filter_info.groups = NULL;
-  
-  if (needed & GTK_RECENT_FILTER_AGE)
-    {
-      filter_info.age = gtk_recent_info_get_age (info);
-      filter_info.contains |= GTK_RECENT_FILTER_AGE;
-    }
-  else
-    filter_info.age = -1;
-  
-  retval = gtk_recent_filter_filter (current_filter, &filter_info);
-  
-  /* this we own */
-  if (filter_info.applications)
-    g_strfreev ((gchar **) filter_info.applications);
-  if (filter_info.groups)
-    g_strfreev ((gchar **) filter_info.groups);
-  
-  return !retval;
-}
-
 /* taken from libeel/eel-strings.c */
 static gchar *
 escape_underscores (const gchar *string)
@@ -894,7 +760,7 @@ gtk_recent_chooser_menu_add_tip (GtkRecentChooserMenu *menu,
                             item,
                             tip_text,
                             NULL);
-      
+
       g_free (path);
       g_free (tip_text);
     }
@@ -926,9 +792,21 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
       
       /* avoid clashing mnemonics */
       if (count <= 10)
-        text = g_strdup_printf ("_%d. %s", count, escaped);
+        /* This is the label format that is used for the first 10 items 
+         * in a recent files menu. The %d is the number of the item,
+         * the %s is the name of the item. Please keep the _ in front
+         * of the number to give these menu items a mnemonic.
+         *
+         * Don't include the prefix "recent menu label|" in the translation.
+         */
+        text = g_strdup_printf (Q_("recent menu label|_%d. %s"), count, escaped);
       else
-        text = g_strdup_printf ("%d. %s", count, escaped);
+        /* This is the format that is used for items in a recent files menu. 
+         * The %d is the number of the item, the %s is the name of the item. 
+         *
+         * Don't include the prefix "recent menu label|" in the translation.
+         */
+        text = g_strdup_printf (Q_("recent menu label|%d. %s"), count, escaped);
       
       item = gtk_image_menu_item_new_with_mnemonic (text);
       
@@ -943,8 +821,11 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
 
   g_free (text);
 
+  /* ellipsize the menu item label, in case the recent document
+   * display name is huge.
+   */
   label = GTK_BIN (item)->child;
-  if (label)
+  if (GTK_IS_LABEL (label))
     {
       gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
       gtk_label_set_max_width_chars (GTK_LABEL (label), priv->label_width);
@@ -956,7 +837,6 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
         
       image = gtk_image_new_from_pixbuf (icon);
       gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-
       g_object_unref (icon);
     }
 
@@ -967,6 +847,45 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
   return item;
 }
 
+static void
+gtk_recent_chooser_menu_insert_item (GtkRecentChooserMenu *menu,
+                                     GtkWidget            *menuitem,
+                                     gint                  position)
+{
+  GtkRecentChooserMenuPrivate *priv = menu->priv;
+  gint real_position;
+
+  if (priv->first_recent_item_pos == -1)
+    {
+      GList *children, *l;
+
+      children = gtk_container_get_children (GTK_CONTAINER (menu));
+
+      for (real_position = 0, l = children;
+           l != NULL;
+           real_position += 1, l = l->next)
+        {
+          GObject *child = l->data;
+          gboolean is_placeholder = FALSE;
+
+          is_placeholder =
+            GPOINTER_TO_INT (g_object_get_data (child, "gtk-recent-menu-placeholder"));
+
+          if (is_placeholder)
+            break;
+        }
+
+      g_list_free (children);
+      priv->first_recent_item_pos = real_position;
+    }
+  else
+    real_position = priv->first_recent_item_pos;
+
+  gtk_menu_shell_insert (GTK_MENU_SHELL (menu), menuitem,
+                         real_position + position);
+  gtk_widget_show (menuitem);
+}
+
 /* removes the items we own from the menu */
 static void
 gtk_recent_chooser_menu_dispose_items (GtkRecentChooserMenu *menu)
@@ -977,12 +896,13 @@ gtk_recent_chooser_menu_dispose_items (GtkRecentChooserMenu *menu)
   for (l = children; l != NULL; l = l->next)
     {
       GtkWidget *menu_item = GTK_WIDGET (l->data);
-      gint mark = 0;
+      gboolean has_mark = FALSE;
       
       /* check for our mark, in order to remove just the items we own */
-      mark = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item),
-                                                 "gtk-recent-menu-mark"));
-      if (mark == 1)
+      has_mark =
+        GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item), "gtk-recent-menu-mark"));
+
+      if (has_mark)
         {
           GtkRecentInfo *info;
           
@@ -997,6 +917,9 @@ gtk_recent_chooser_menu_dispose_items (GtkRecentChooserMenu *menu)
         }
     }
 
+  /* recalculate the position of the first recent item */
+  menu->priv->first_recent_item_pos = -1;
+
   g_list_free (children);
 }
 
@@ -1005,7 +928,9 @@ typedef struct
   GList *items;
   gint n_items;
   gint loaded_items;
+  gint displayed_items;
   GtkRecentChooserMenu *menu;
+  GtkWidget *placeholder;
 } MenuPopulateData;
 
 static gboolean
@@ -1017,8 +942,6 @@ idle_populate_func (gpointer data)
   gboolean retval;
   GtkWidget *item;
 
-  GDK_THREADS_ENTER ();
-
   pdata = (MenuPopulateData *) data;
   priv = pdata->menu->priv;
 
@@ -1029,79 +952,34 @@ idle_populate_func (gpointer data)
       pdata->items = gtk_recent_chooser_get_items (GTK_RECENT_CHOOSER (pdata->menu));
       if (!pdata->items)
         {
-          item = gtk_menu_item_new_with_label (_("No items found"));
-          gtk_widget_set_sensitive (item, FALSE);
-      
-          /* we also mark this item, so that it gets removed when rebuilding
-           * the menu on the next map event
-           */
-          g_object_set_data (G_OBJECT (item), "gtk-recent-menu-mark",
-      			     GINT_TO_POINTER (1));
-      
-          gtk_menu_shell_prepend (GTK_MENU_SHELL (pdata->menu), item);
-          gtk_widget_show (item);
-
-	  /* no items: add a placeholder menu */
-          GDK_THREADS_LEAVE ();
+          /* show the placeholder here */
+          gtk_widget_show (pdata->placeholder);
+          pdata->displayed_items = 1;
 
 	  return FALSE;
 	}
       
-      /* reverse the list */
-      pdata->items = g_list_reverse (pdata->items);
-      
       pdata->n_items = g_list_length (pdata->items);
       pdata->loaded_items = 0;
     }
 
   info = g_list_nth_data (pdata->items, pdata->loaded_items);
-
-  /* skip non-local items on request */
-  if (priv->local_only &&
-      !gtk_recent_info_is_local (info))
-    {
-      goto check_and_return;
-    }
-      
-  /* skip private items on request */
-  if (!priv->show_private &&
-      gtk_recent_info_get_private_hint (info))
-    {
-      goto check_and_return;
-    }
-
-  /* skip non-existing items on request */
-  if (!priv->show_not_found &&
-      !gtk_recent_info_exists (info))
-    {
-      goto check_and_return;
-    }
-  /* filter items based on the currently set filter object */
-  if (get_is_recent_filtered (pdata->menu, info))
-    {
-      goto check_and_return;
-    }
-
   item = gtk_recent_chooser_menu_create_item (pdata->menu,
                                               info,
-					      pdata->loaded_items);
+					      pdata->displayed_items);
   if (!item)
     goto check_and_return;
       
   gtk_recent_chooser_menu_add_tip (pdata->menu, info, item);
-      
-  /* FIXME
-   *
-   * We should really place our items taking into account user
-   * defined menu items; this would also remove the need of
-   * reverting the scan order.
-   */
-  gtk_menu_shell_prepend (GTK_MENU_SHELL (pdata->menu), item);
-  gtk_widget_show (item);
+  gtk_recent_chooser_menu_insert_item (pdata->menu, item,
+                                       pdata->displayed_items);
+  
+  pdata->displayed_items += 1;
       
   /* mark the menu item as one of our own */
-  g_object_set_data (G_OBJECT (item), "gtk-recent-menu-mark",
-      		     GINT_TO_POINTER (1));
+  g_object_set_data (G_OBJECT (item),
+                     "gtk-recent-menu-mark",
+      		     GINT_TO_POINTER (TRUE));
       
   /* attach the RecentInfo object to the menu item, and own a reference
    * to it, so that it will be destroyed with the menu item when it's
@@ -1124,14 +1002,21 @@ check_and_return:
   else
     retval = TRUE;
 
-  GDK_THREADS_LEAVE ();
-
   return retval;
 }
 
 static void
 idle_populate_clean_up (gpointer data)
 {
+  MenuPopulateData *pdata = data;
+
+  /* show the placeholder in case no item survived
+   * the filtering process in the idle loop
+   */
+  if (!pdata->displayed_items)
+    gtk_widget_show (pdata->placeholder);
+  g_object_unref (pdata->placeholder);
+
   g_slice_free (MenuPopulateData, data);
 }
 
@@ -1139,6 +1024,7 @@ static void
 gtk_recent_chooser_menu_populate (GtkRecentChooserMenu *menu)
 {
   MenuPopulateData *pdata;
+  GtkRecentChooserMenuPrivate *priv = menu->priv;
 
   if (menu->priv->populate_id)
     return;
@@ -1147,17 +1033,20 @@ gtk_recent_chooser_menu_populate (GtkRecentChooserMenu *menu)
   pdata->items = NULL;
   pdata->n_items = 0;
   pdata->loaded_items = 0;
+  pdata->displayed_items = 0;
   pdata->menu = menu;
+  pdata->placeholder = g_object_ref (priv->placeholder);
 
-  menu->priv->icon_size = get_icon_size_for_widget (GTK_WIDGET (menu));
+  priv->icon_size = get_icon_size_for_widget (GTK_WIDGET (menu));
   
-  /* dispose our menu items first */
+  /* remove our menu items first and hide the placeholder */
   gtk_recent_chooser_menu_dispose_items (menu);
+  gtk_widget_hide (priv->placeholder);
   
-  menu->priv->populate_id = g_idle_add_full (G_PRIORITY_HIGH_IDLE + 30,
-  					     idle_populate_func,
-					     pdata,
-					     idle_populate_clean_up);
+  priv->populate_id = gdk_threads_add_idle_full (G_PRIORITY_HIGH_IDLE + 30,
+  					         idle_populate_func,
+					         pdata,
+                                                 idle_populate_clean_up);
 }
 
 /* bounce activate signal from the recent menu item widget 
@@ -1191,7 +1080,16 @@ set_recent_manager (GtkRecentChooserMenu *menu,
   if (priv->manager)
     {
       if (priv->manager_changed_id)
-        g_signal_handler_disconnect (priv->manager, priv->manager_changed_id);
+        {
+          g_signal_handler_disconnect (priv->manager, priv->manager_changed_id);
+          priv->manager_changed_id = 0;
+        }
+
+      if (priv->populate_id)
+        {
+          g_source_remove (priv->populate_id);
+          priv->populate_id = 0;
+        }
 
       priv->manager = NULL;
     }
@@ -1203,10 +1101,8 @@ set_recent_manager (GtkRecentChooserMenu *menu,
   
   if (priv->manager)
     priv->manager_changed_id = g_signal_connect (priv->manager, "changed",
-      						 G_CALLBACK (manager_changed_cb),
-      						 menu);
-  /* (re)populate the menu */
-  gtk_recent_chooser_menu_populate (menu);
+                                                 G_CALLBACK (manager_changed_cb),
+                                                 menu);
 }
 
 static gint
@@ -1327,7 +1223,7 @@ gtk_recent_chooser_menu_get_show_numbers (GtkRecentChooserMenu *menu)
  *
  * Sets whether a number should be added to the items of @menu.  The
  * numbers are shown to provide a unique character for a mnemonic to
- * be used inside the menu item's label.  Only the first the items
+ * be used inside ten menu item's label.  Only the first the items
  * get a number to avoid clashes.
  *
  * Since: 2.10
