@@ -35,9 +35,7 @@
 #endif
 
 #include "gtksearchenginesimple.h"
-
-#define XDG_PREFIX _gtk_xdg
-#include "xdgmime/xdgmime.h"
+#include "gtkprivate.h"
 
 #include <string.h>
 
@@ -50,7 +48,6 @@ typedef struct
   GtkSearchEngineSimple *engine;
   
   gchar *path;
-  GList *mime_types;
   gchar **words;
   GList *found_list;
   
@@ -99,7 +96,7 @@ search_thread_data_new (GtkSearchEngineSimple *engine,
   
   data = g_new0 (SearchThreadData, 1);
   
-  data->engine = engine;
+  data->engine = g_object_ref (engine);
   uri = _gtk_query_get_location (query);
   if (uri != NULL) 
     {
@@ -115,18 +112,15 @@ search_thread_data_new (GtkSearchEngineSimple *engine,
   g_free (text);
   g_free (lower);
   
-  data->mime_types = _gtk_query_get_mime_types (query);
-  
   return data;
 }
 
 static void 
 search_thread_data_free (SearchThreadData *data)
 {
+  g_object_unref (data->engine);
   g_free (data->path);
   g_strfreev (data->words);
-  g_list_foreach (data->mime_types, (GFunc)g_free, NULL);
-  g_list_free (data->mime_types);
   g_free (data);
 }
 
@@ -187,7 +181,7 @@ send_batch (SearchThreadData *data)
       hits = g_new (SearchHits, 1);
       hits->uris = data->uri_hits;
       hits->thread_data = data;
-      g_idle_add (search_thread_add_hits_idle, hits);
+      gdk_threads_add_idle (search_thread_add_hits_idle, hits);
     }
   data->uri_hits = NULL;
 }
@@ -204,10 +198,9 @@ search_visit_func (const char        *fpath,
   SearchThreadData *data;
   gint i;
   const gchar *name; 
-  gchar *lower_name, *mime_type;
+  gchar *lower_name;
   gchar *uri;
   gboolean hit;
-  GList *l;
   gboolean is_hidden;
   
   data = (SearchThreadData*)g_static_private_get (&search_thread_data);
@@ -245,22 +238,6 @@ search_visit_func (const char        *fpath,
       g_free (lower_name);
     }
 
-  if (hit && data->mime_types != NULL) 
-    {
-      hit = FALSE;
-      mime_type = xdg_mime_get_mime_type_for_file (fpath, (struct stat *)sb);
-      for (l = data->mime_types; l != NULL; l = l->next) 
-	{
-	  if (strcmp (mime_type, l->data) == 0) 
-	    {
-	      hit = TRUE;
-	      break;
-	    }
-	}
-
-      g_free (mime_type);
-    }
-
   if (hit) 
     {
       uri = g_filename_to_uri (fpath, NULL, NULL);
@@ -301,7 +278,7 @@ search_thread_func (gpointer user_data)
 
   send_batch (data);
   
-  g_idle_add (search_thread_done_idle, data);
+  gdk_threads_add_idle (search_thread_done_idle, data);
 #endif /* HAVE_FTW_H */
   
   return NULL;
