@@ -1,7 +1,7 @@
 /* GTK - The GIMP Toolkit
  * gtkprintbackendcups.h: Default implementation of GtkPrintBackend 
  * for the Common Unix Print System (CUPS)
- * Copyright (C) 2003, Red Hat, Inc.
+ * Copyright (C) 2006, 2007 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -426,13 +426,15 @@ gtk_print_backend_cups_print_stream (GtkPrintBackend         *print_backend,
 				  NULL,
 				  cups_printer->device_uri);
 
-  gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
+  gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, 
+                                   IPP_TAG_URI, "printer-uri",
                                    NULL, cups_printer->printer_uri);
 
   title = gtk_print_job_get_title (job);
   if (title)
-    gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, IPP_TAG_NAME, "job-name", NULL,
-                                     title);
+    gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, 
+                                     IPP_TAG_NAME, "job-name", 
+                                     NULL, title);
 
   gtk_print_settings_foreach (settings, add_cups_options, request);
   
@@ -625,8 +627,7 @@ cups_dispatch_watch_finalize (GSource *source)
       dispatch->backend = NULL;
     }
 
-  if (dispatch->data_poll != NULL)
-    g_free (dispatch->data_poll);
+  g_free (dispatch->data_poll);
 }
 
 static GSourceFuncs _cups_dispatch_watch_funcs = {
@@ -893,7 +894,7 @@ static void
 cups_request_job_info (CupsJobPollData *data)
 {
   GtkCupsRequest *request;
-  gchar *printer_uri;
+  gchar *job_uri;
 
   request = gtk_cups_request_new (NULL,
                                   GTK_CUPS_POST,
@@ -902,10 +903,10 @@ cups_request_job_info (CupsJobPollData *data)
 				  NULL,
 				  NULL);
 
-  printer_uri = g_strdup_printf ("ipp://localhost/jobs/%d", data->job_id);
+  job_uri = g_strdup_printf ("ipp://localhost/jobs/%d", data->job_id);
   gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, IPP_TAG_URI,
-                                   "job-uri", NULL, printer_uri);
-  g_free (printer_uri);
+                                   "job-uri", NULL, job_uri);
+  g_free (job_uri);
 
   cups_request_execute (data->print_backend,
                         request,
@@ -1024,13 +1025,13 @@ cups_request_printer_list_cb (GtkPrintBackendCups *cups_backend,
 
       while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER)
       {
-        if (!strcmp (attr->name, "printer-name") &&
+        if (strcmp (attr->name, "printer-name") == 0 &&
 	    attr->value_tag == IPP_TAG_NAME)
 	  printer_name = attr->values[0].string.text;
-	else if (!strcmp (attr->name, "printer-uri-supported") &&
+	else if (strcmp (attr->name, "printer-uri-supported") == 0 &&
 		 attr->value_tag == IPP_TAG_URI)
 	  printer_uri = attr->values[0].string.text;
-	else if (!strcmp (attr->name, "member-uris") &&
+	else if (strcmp (attr->name, "member-uris") == 0 &&
 		 attr->value_tag == IPP_TAG_URI)
 	  member_uris = attr->values[0].string.text;
         else if (strcmp (attr->name, "printer-location") == 0)
@@ -1113,14 +1114,14 @@ cups_request_printer_list_cb (GtkPrintBackendCups *cups_backend,
 			resource);
 #endif
 
-          if (!strncmp (resource, "/printers/", 10))
+          if (strncmp (resource, "/printers/", 10) == 0)
 	    {
 	      cups_printer->ppd_name = g_strdup (resource + 10);
               GTK_NOTE (PRINTING,
                         g_print ("CUPS Backend: Setting ppd name '%s' for printer/class '%s'\n", cups_printer->ppd_name, printer_name));
             }
 
-	  gethostname (uri, sizeof(uri));
+	  gethostname (uri, sizeof (uri));
 	  if (strcasecmp (uri, hostname) == 0)
 	    strcpy (hostname, "localhost");
 
@@ -1172,7 +1173,6 @@ cups_request_printer_list_cb (GtkPrintBackendCups *cups_backend,
 
       /* The ref is held by GtkPrintBackend, in add_printer() */
       g_object_unref (printer);
-
       
       if (attr == NULL)
         break;
@@ -1187,8 +1187,7 @@ cups_request_printer_list_cb (GtkPrintBackendCups *cups_backend,
       list_has_changed = TRUE;
     }
   
- done:
-
+done:
   if (list_has_changed)
     g_signal_emit_by_name (backend, "printer-list-changed");
   
@@ -1248,7 +1247,7 @@ cups_get_printer_list (GtkPrintBackend *backend)
   if (cups_backend->list_printers_poll == 0)
     {
       cups_request_printer_list (cups_backend);
-      cups_backend->list_printers_poll = g_timeout_add (3000,
+      cups_backend->list_printers_poll = gdk_threads_add_timeout (3000,
                                                         (GSourceFunc) cups_request_printer_list,
                                                         backend);
     }
@@ -2300,7 +2299,7 @@ cups_printer_get_options (GtkPrinter           *printer,
         {
           gchar *custom_name;
 
-	  custom_name = g_strdup_printf ("Custom.%2fx%.2f",
+	  custom_name = g_strdup_printf (_("Custom %.2fx%.2f"),
 					 gtk_paper_size_get_width (paper_size, GTK_UNIT_POINTS),
 					 gtk_paper_size_get_height (paper_size, GTK_UNIT_POINTS));
           strncpy (option->defchoice, custom_name, PPD_MAX_NAME);
@@ -2885,5 +2884,6 @@ cups_printer_get_capabilities (GtkPrinter *printer)
   return
     GTK_PRINT_CAPABILITY_COPIES |
     GTK_PRINT_CAPABILITY_COLLATE |
-    GTK_PRINT_CAPABILITY_REVERSE;
+    GTK_PRINT_CAPABILITY_REVERSE |
+    GTK_PRINT_CAPABILITY_NUMBER_UP;
 }
